;
;       scrlltxt.asm
;       TI-82 CrASH 1.6 version
;
;       Scrolltext demo
;
; Last modified: 04/24/2002 mmddyy 18:41 by Burkart
;
; size: 1933 byte (549 byte code+data, 1384 byte demo text)
;

#include "CRASH82.INC"

NO_OF_LINES = 8         ; has to be 1..8, only tested with 8, has to be 8 for correct scroll bars
CHARS_EACH_LINE = 15    ; has to be 1..16, should be 15, 16 results in faulty CR+LF (could be adjusted)
SPACE_CHAR = ' '        ; should be ' '

        .db "Scrolltext Demo",0

home:
        xor     a
refresh_display:
        ld      (var_top_line), a
        call    disp_scrlltxt

main_key_loop:
        halt                                            ; save power
        call    GET_KEY                                 ; get keypad data
        cp      G_DOWN                                  ; "DOWN" -> scroll down
        jr      z, scroll_down
        cp      G_UP                                    ; "UP" -> scroll up
        jr      z, scroll_up
        cp      G_DEL                                   ; "DEL" -> page up
        jr      z, page_up
        cp      G_STAT                                  ; "STAT" -> page down
        jr      z, page_down
        cp      G_2ND                                   ; "2nd" -> home
        jr      z, home
        cp      G_ALPHA                                 ; "ALPHA" -> end
        jr      z, end
        cp      G_MODE                                  ; "MODE" -> exit to CrASH
        ret     z
        jr      main_key_loop                           ; other/no key -> loop again

scroll_down:
        ld      a, (var_top_line)
        add     a, NO_OF_LINES - 1
        ld      hl, var_current_line                    ; make sure, there is another line
        cp      (hl)
        jr      z, main_key_loop
        sub     NO_OF_LINES - 2
        jr      refresh_display

scroll_up:
        ld      a, (var_top_line)                       ; already at top most line
        or      a
        jr      z, main_key_loop                        ; yes, key loop again
        dec     a                                       ; no, scroll up and refresh display
        jr      refresh_display

page_up:
        ld      a, (var_top_line)                       ; move top_line up NO_OF_LINES lines (1 screen)
        sub     NO_OF_LINES
        jr      c, home                                 ; result is "negative" line? then goto first line
        jr      refresh_display                         ; otherwise display with calculated top line

page_down:
        ld      b, NO_OF_LINES
        ld      a, (var_top_line)
        add     a, NO_OF_LINES - 2
        ld      hl, var_current_line
page_down_loop:
        inc     a
        cp      (hl)
        jr      z, end
        djnz    page_down_loop
        sub     NO_OF_LINES - 2
        jr      refresh_display

end:
        ld      a, (var_current_line)                   ; get number of lines as temporary top_line
        ld      b, NO_OF_LINES-1                        ; "scroll up" NO_OF_LINES-1 times
end_loop:
        or      a                                       ; refresh display in case we're already at top most line
        jr      z, refresh_display
        dec     a
        djnz    end_loop
        jr      refresh_display

;------------------------------------------------
; call disp_scrlltxt
;------------------------------------------------
; (DESCRIPTION)
;------------------------------------------------
; input: (...)
;------------------------------------------------
; modified: (...)
;------------------------------------------------

disp_scrlltxt:
        res     2, (IY+$0D)                             ; disable auto scrolling

        ld      hl, $0000                               ; move cursor to upper left corner
        ld      (CURSOR_POS), hl

        xor     a                                       ; reset var_chr_processed through var_current_line to 0
        ld      hl, var_chr_processed
        ld      b, 3
reset_var_loop:
        ld      (hl), a
        inc     hl
        djnz    reset_var_loop

next_line:
        ld      hl, TEXT_LEN                            ; check, if more than CHARS_EACH_LINE chars are left
        ld      de, (var_chr_processed)                 ; unprocessed, would be the case if
        push    de ; push (var_chr_processed)           ; CHARS_EACH_LINE < [TEXT_LEN - (var_chr_processed)]
        scf
        ccf
        sbc     hl, de
        ex      de, hl
        ld      hl, CHARS_EACH_LINE
        scf
        ccf
        sbc     hl, de
        jr      c, more_than_chars_each_line_chars_left

        ; [<= CHARS_EACH_LINE] chars left

        ld      a, (var_top_line)                       ; only display line if (var_current_line) >= (var_top_line)
        ld      hl, var_current_line
        cp      (hl)
        jr      z, do_display_1
        jr      nc, dont_display_1
do_display_1b:
        add     a, NO_OF_LINES - 1                      ; and only display line if
        cp      (hl)                                    ; (var_current_line) <= (var_top_line)+NO_OF_LINES-1
        jr      c, dont_display_1

        ld      hl, str_text                            ; directly display remaining chars
        pop     de ; pop (var_chr_processed)
        add     hl, de
        ROM_CALL(D_ZT_STR)
        jr      end_disp_scrlltxt                       ; clean up, then end routine

dont_display_1:
        pop     de                                      ; clean up stack
        jr      end_disp_scrlltxt                       ; clean up, then end routine

do_display_1:
        push    af                                      ; TODO: check if push/pop is necessary
        push    hl
        ROM_CALL(CLEARLCD)                              ; clear LCD
        pop     hl
        pop     af
        jr      do_display_1b

more_than_chars_each_line_chars_left:
        ; more than CHARS_EACH_LINE chars left

        pop     de ; pop (var_chr_processed)            ; check, if SPACE_CHAR after next CHARS_EACH_LINE chars
        ld      hl, str_text                            ; would be the case if
        add     hl, de                                  ; (string+(var_chr_processsed)+CHARS_EACH_LINE) == SPACE_CHAR
        push    hl ; push str_text+(var_chr_processed)
        ld      de, CHARS_EACH_LINE
        add     hl, de
        ld      a, SPACE_CHAR
        cp      (hl)
        jr      nz, search_for_space_char

        ; SPACE_CHAR after next CHARS_EACH_LINE chars

        pop     hl ; pop str_text+(var_chr_processed)   ; display next CHARS_EACH_LINE chars
        ld      a, CHARS_EACH_LINE
        call    disp_a_chars

        call    cr_lf                                   ; CR + LF

        ld      hl, (var_chr_processed)                 ; update (var_chr_processed) by
        ld      bc, CHARS_EACH_LINE+1                   ; (var_chr_processed) += CHARS_EACH_LINE + 1
        add     hl, bc
        ld      (var_chr_processed), hl

        ld      a, (var_current_line)                   ; update (var_current_line) by
        inc     a                                       ; (var_current_line)++
        ld      (var_current_line), a

        jr      next_line                               ; process next line

search_for_space_char:
        ; search for SPACE_CHAR in next CHARS_EACH_LINE chars:

        pop     hl ; pop str_text+(var_chr_processed)
        push    hl ; push str_text+(var_chr_processed)
        ld      bc, CHARS_EACH_LINE - 1
        add     hl, bc
        ld      a, SPACE_CHAR
        cpdr
        jr      z, space_char_found

        ; SPACE_CHAR not found in next CHARS_EACH_LINE chars
        ; -> display CHARS_EACH_LINE chars

        pop     hl ; pop str_text+(var_chr_processed)
        ld      a, CHARS_EACH_LINE
        call    disp_a_chars
        
        call    cr_lf                                   ; CR + LF

        ld      hl, (var_chr_processed)                 ; update (var_chr_processed) by
        ld      bc, CHARS_EACH_LINE                     ; (var_chr_processed) += CHARS_EACH_LINE
        add     hl, bc
        ld      (var_chr_processed), hl

inc_current_line_and_next_line:
        ld      a, (var_current_line)                   ; (var_current_line)++
        inc     a
        ld      (var_current_line), a

        jp      next_line

space_char_found:
        ; SPACE_CHAR found in next CHARS_EACH_LINE chars

        ld      hl, (var_chr_processed)                 ; (var_chr_processed) += ... (bc) + 1
        add     hl, bc
        inc     hl
        inc     hl
        ld      (var_chr_processed), hl

        pop     hl ; pop str_text+(var_chr_processed)
        ld      a, c
        inc     a
        call    disp_a_chars

        call    cr_lf                                   ; adjust cursor (CR+LF)

        jr      inc_current_line_and_next_line          ; (var_current_line)++ and jr next_line

end_disp_scrlltxt:
        ; display scroll bars:

        ld      hl, CURSOR_ROW                          ; grey out complete scroll bar
        ld      (hl), $00
        inc     hl
        ld      b, 8
        ld      a, $E1
scroll_bar_loop1:
        ld      (hl), 15
        ROM_CALL(TX_CHARPUT)
        djnz    scroll_bar_loop1

        ; display black field(s) in scroll bar:
        ld      a, (var_top_line)
        ld      l, a
        ld      h, 0
        ld      d, h
        inc     l
        sla     l
        rl      h
        sla     l
        rl      h
        sla     l
        rl      h
        ld      a, (var_current_line)
        ld      e, a
        push    de
        call    DIV_HL_DE_S
        ld      a, l
        ld      (CURSOR_ROW), a
        pop     de
        ld      hl, NO_OF_LINES*8
        call    DIV_HL_DE_S
        ld      b, l
        inc     b
        ld      a, 8
        cp      b
        jr      nc, scroll_bar_display2
        ld      b, a
scroll_bar_display2:
        ld      a, $D9
        ld      hl, CURSOR_COL
scroll_bar_loop2:
        ld      (hl), 15
        ROM_CALL(TX_CHARPUT)
        djnz    scroll_bar_loop2


        ; clean up:
        set     2, (IY+$0D)                             ; enable auto scrolling

        ret                                             ; RETurn to caller of disp_scrlltxt

;------------------------------------------------
; call cr_lf
;------------------------------------------------
; adjust cursor (CR+LF)
;------------------------------------------------
; input: none
;------------------------------------------------
; modified: af, hl
;------------------------------------------------

cr_lf:
        ld      a, (var_top_line)                       ; only CR+LF if (var_current_line) >= (var_top_line)
        ld      hl, var_current_line
        cp      (hl)
        jr      z, do_display_3
        ret     nc
do_display_3:
        add     a, NO_OF_LINES - 1                      ; and only CR+LF if
        cp      (hl)                                    ; (var_current_line) <= (var_top_line)+NO_OF_LINES-1
        ret     c

        ld      hl, CURSOR_ROW                          ; (CURSOR_ROW)++     (LF)
        inc     (hl)
        inc     hl                                      ; (CURSOR_COL) := 0  (CR)
        xor     a
        ld      (hl), a
        ret                                             ; RETurn to caller

;------------------------------------------------
; call disp_a_chars
;------------------------------------------------
; displays A chars
;------------------------------------------------
; input: HL - pointer to string
;        A  - number of chars to display (1..16)
;------------------------------------------------
; modified: hl, bc=0, de, more due to D_LT_STR???
;------------------------------------------------

disp_a_chars:
        push    af                                      ; save registers used during check
        push    hl

        ld      a, (var_top_line)                       ; only display line if (var_current_line) >= (var_top_line)
        ld      hl, var_current_line
        cp      (hl)
        jr      z, do_display_2
        jr      nc, disp_a_chars_exit
        add     a, NO_OF_LINES - 1                      ; and only display line if
        cp      (hl)                                    ; (var_current_line) <= (var_top_line)+NO_OF_LINES-1
        jr      c, disp_a_chars_exit

do_display_2b:
        pop     hl                                      ; restore registers used during check
        pop     af

        ld      (buf_disp_a_chars), a                   ; copy chars to display buffer
        ld      de, buf_disp_a_chars+1
        ld      b, 0
        ld      c, a
        ldir
        ld      hl, buf_disp_a_chars                    ; display chars in buffer
        ROM_CALL(D_LT_STR)
        ret                                             ; RETurn to caller

disp_a_chars_exit:
        pop     hl
        pop     af
        ret

do_display_2:
        ROM_CALL(CLEARLCD)
        jr      do_display_2b


#include "div_hl_de_s.h"

;================================================
; variables, buffers, strings
;================================================

var_top_line:
        .db     0
var_chr_processed:
        .dw     0
var_current_line:
        .db     0
buf_disp_a_chars:
        .db     16,"1234567890123456"
str_text:
        .db     "Beastie Boys / Just a test: "
        .db     "It's like a flag at half mast as frames click fast / "
        .db     "Not a thing will last as past is past / "
        .db     "It's like stacks of thoughts that got played and worn / "
        .db     "And used over and over till they were tired and torn / "
        .db     "And like a broken clock that can't tell time / "
        .db     "And like a thick ass book that's filled with wack rhymes / "
        .db     "It's like a scorching blaze that burned the sand / "
        .db     "And like a band that planned and planned and planned / "
        .db     "And flew down like a raven in the dark of night / "
        .db     "And snatched up the worm helpless to fight / "
        .db     "And brought it back to the nest singing microphone check / "
        .db     "One two one two this is just a test / "
        .db     "One two one two this is just a test / "
        .db     "One two one two this is just a test / "
        .db     "It's like sand in one hand you can't hold for long / "
        .db     "And like scheming on a plan that goes all wrong / "
        .db     "It's like fudge and caramel they're not the same / "
        .db     "And it's a shame all you talk is game / "
        .db     "You act like a diamond waiting to be set / "
        .db     "In a gold ring, as if, I bet / "
        .db     "As time goes by in this give and take / "
        .db     "As long as I learn I will make mistakes / "
        .db     "Now, what do I want? What do I need? "
        .db     "Why do I want it? What's in it for me? "
        .db     "It's the imagery of technology / "
        .db     "Is what you get is what you see / "
        .db     "Don't worry your mind / "
        .db     "When you give it your best / "
        .db     "One two one two this is just a test / "
        .db     "One two one two this is just a test / "
        .db     "One two one two this is just a test / "
        .db     "C'mon (15x)",0
str_text_end:
TEXT_LEN = (str_text_end - str_text) - 1        ; number of chars, not counting trailing ASCII-Z
